class User < ApplicationRecord
  attr_accessor :password
  has_many :borrows
  has_many :current_borrows, -> { where status: :borrowed }, class_name: 'Borrow'
  has_many :historic_borrows, -> { where return_time: nil }, class_name: 'Borrow'
  has_many :items, through: :borrows
  has_many :current_items, through: :current_borrows, source: :user
  has_many :histotic_items, through: :historic_borrows, source: :user

  before_save :encrypt_password

  validates :email, {
    presence: true, uniqueness: true
  }
  validates :name, presence: true
  validates :password, confirmation: true, if: :password_required?

  def self.encrypt(pass, salt)
    Digest::SHA1.hexdigest(pass+salt)
  end

  def encrypt_password
    return if password.blank?
    if new_record?
      self.salt = SecureRandom.hex(16)
    end
    self.encrypted_password = User.encrypt(self.password, self.salt)
  end

  def self.authenticate(email, pass)
    user = User.where(email: email).first # User.find_by email: email # User.find_by_email email
    user && user.authenticated?(pass) ? user : nil
  end

  def authenticated?(pass)
    self.encrypted_password == User.encrypt(pass, self.salt)
  end

  def password_required?
    self.new_record? || !password.blank?
  end
end
